#define SIZE 256
#define MAXCOUNT 200

union color
{
  struct
  {
    unsigned char r;
    unsigned char g;
    unsigned char b;
    unsigned char a;
  };
  unsigned int c;
} color[SIZE][SIZE];

typedef struct vec2
{
  float x;
  float y;
} vec2;

static inline union color colorize(unsigned int a)
{
    int i = ((float)a / (float)MAXCOUNT) * 6.0;
    float f = (((float)a / (float)MAXCOUNT) * 6.0) - i;
    float q = 1.0 - f;
    float t = 1.0 - ((1.0 - f) * 1.0);

    switch(i % 6)
    {
        case 0: return (union color){ .r = 0xFF    , .g = t * 0xFF, .b = 0x00    , .a = 0xFF };
        case 1: return (union color){ .r = q * 0xFF, .g = 0xFF    , .b = 0x00    , .a = 0xFF };
        case 2: return (union color){ .r = 0x00    , .g = 0xFF    , .b = t * 0xFF, .a = 0xFF };
        case 3: return (union color){ .r = 0x00    , .g = q * 0xFF, .b = 0xFF    , .a = 0xFF };
        case 4: return (union color){ .r = t * 0xFF, .g = 0x00    , .b = 0xFF    , .a = 0xFF };
        case 5: return (union color){ .r = 0xFF    , .g = 0x00    , .b = q * 0xFF, .a = 0xFF };
        default: return (union color){ .r = 0, .g = 0, .b = 0, .a = 0xFF };
    };
}

int game(float time);
int game(float time)
{
  unsigned int cycle = (time * 0.05) * MAXCOUNT;
  for (int y = 0;y < SIZE;y++)
  {
    for (int x = 0;x < SIZE;x++)
    {
      vec2 c = (vec2){ .x = (((((float)x / SIZE) - 0.5) / time) * 2.0) - 0.39, .y = (((((float)y / SIZE) - 0.5) / time) * 2.0) + 0.645 };
      vec2 p = { 0 };
      unsigned int count = 0;

      while (p.x * p.x + p.y * p.y < 2*2 && count < MAXCOUNT)
      {
        float t = p.x * p.x - p.y * p.y + c.x;
        p.y = 2 * p.x * p.y + c.y;
        p.x = t;
        count++;
      }
      if (count == MAXCOUNT)
        color[y][x].c = 0x00;
      else
        color[y][x] = colorize(count+cycle);
    }
  }

  return 0;
}
